AddonName, a = ...
if a.BuildFail(50000) then return end
local L = a.Localize
local s = SpellFlashAddon
local x = s.UpdatedVariables

-- SpellFlash API:
-- http://wow.curseforge.com/addons/spellflash/pages/api/

-- Updated variable references:
-- http://wow.curseforge.com/addons/spellflash/pages/api/#w-local-x-spell-flash-addon-updated-variables
-- x.Enemy, x.ActiveEnemy, x.NoCC, x.InInstance, x.InstanceType, x.PetAlive, x.PetActiveEnemy, x.PetNoCC, x.Lag, x.DoubleLag, x.ThreatPercent, x.EnemyDetected, x.ShouldPermanentBuff, x.ShouldTemporaryBuff

-- Spell table references:
-- http://wow.curseforge.com/addons/spellflash/pages/api/#w-s-castable

local SpellPowerMultiplier = {
	1459--[[Arcane Brilliance]],
	77747--[[Burning Wrath]],
	109773--[[Dark Intent]],
}

local MagicVulnerability = {
	1490--[[Curse of the Elements]],
	58410--[[Master Poisoner]],
	116202--[[Aura of the Elements]],
}

a.spells["Auto Attack"] = {
	ID = 6603--[[Auto Attack]],
	EnemyTargetNeeded = true,
	Melee = 1,
	NotIfActive = 1,
	Check = function(z)
		z.FlashColor = s.If(x.ActiveEnemy, "Green", "Pink")
		return x.NoCC
	end,
}

a.spells["Shoot"] = {
	ID = 3018--[[Shoot]],
	EnemyTargetNeeded = true,
	NotIfActive = 1,
	Check = function(z)
		z.FlashColor = s.If(x.ActiveEnemy, "Green", "Pink")
		return x.NoCC and not s.Shooting()
	end,
}

a.spells["Pet Attack"] = {
	EnemyTargetNeeded = true,
	Override = function(z)
		if x.NoCC and x.PetAlive and x.PetActiveEnemy and not s.SameTargetAsPet() then
			s.FlashPet(PET_ACTION_ATTACK)
		end
	end,
	FlashColor = "Pink",
	NoGCD = true,
}

a.spells["Pet Assist"] = {
	Override = function(z)
		if x.ActiveEnemy and x.PetAlive and not x.PetActiveEnemy then
			s.FlashPet(PET_MODE_ASSIST)
		end
	end,
	FlashColor = "Pink",
	NoGCD = true,
}

a.spells["Dark Intent"] = {
	ID = 109773--[[Dark Intent]],
	Buff = SpellPowerMultiplier,
	EarlyRefresh = 10,
	NoRangeCheck = 1,
	BuffUnit = "raid",
	FlashColor = "Green",
}

a.spells["Curse of the Elements"] = {
	ID = 1490--[[Curse of the Elements]],
	Debuff = MagicVulnerability,
	EarlyRefresh = 5,
	NotWhileMoving = false,
	EnemyTargetNeeded = true,
	Check = function(z)
		return x.NoCC
	end,
}

a.spells["Summon Felhunter ooc"] = {
	ID = 691--[[Summon Felhunter]],
	Check = function(z)
		return not x.PetAlive
	end,
	FlashID = {691--[[Summon Felhunter]], 112869--[[Summon Observer]], 10--[[Summon Demon]]}
}

a.spells["Summon Felguard ooc"] = {
	ID = 30146--[[Summon Felguard]],
	Check = function(z)
		return not x.PetAlive
	end,
	FlashID = {30146--[[Summon Felguard]], 112870--[[Summon Wrathguard]], 10--[[Summon Demon]]}
}

a.spells["Dark Soul: Misery"] = {
	ID = 113860--[[Dark Soul: Misery]],
	NotWhileMoving = false,
	Check = function(z)
		return x.ShouldTemporaryBuff
	end,
	FlashColor = "Pink",
	NoGCD = true,
	Continue = true,
}

a.spells["Dark Soul: Knowledge"] = {
	ID = 113861--[[Dark Soul: Knowledge]],
	NotWhileMoving = false,
	Check = function(z)
		return x.ShouldTemporaryBuff and s.Power("player", SPELL_POWER_DEMONIC_FURY) >= 600
	end,
	FlashColor = "Pink",
	NoGCD = true,
	Continue = true,
}

a.spells["Dark Soul: Instability"] = {
	ID = 113858--[[Dark Soul: Instability]],
	NotWhileMoving = false,
	Check = function(z)
		return x.ShouldTemporaryBuff
	end,
	FlashColor = "Pink",
	NoGCD = true,
	Continue = true,
}

a.spells["Grimoire: Felhunter"] = {
	ID = 108501--[[Grimoire of Service]],
	Check = function(z)
		return x.ActiveEnemy and s.HasTalent(108501--[[Grimoire of Service]])
	end,
	FlashID = {111897--[[Grimoire: Felhunter]], 108501--[[Grimoire of Service]]},
}

a.spells["Grimoire: Felguard"] = {
	ID = 111898--[[Grimoire: Felguard]],
	Check = function(z)
		return x.ActiveEnemy and s.HasTalent(108501--[[Grimoire of Service]])
	end,
	FlashID = {111898--[[Grimoire: Felguard]], 108501--[[Grimoire of Service]]},
}

a.spells["Grimoire of Sacrifice"] = {
	ID = 108503--[[Grimoire of Sacrifice]],
	Check = function(z)
		return x.PetAlive and x.ActiveEnemy and s.HasTalent(108503--[[Grimoire of Sacrifice]])
	end,
	FlashColor = "Yellow",
	NoGCD = true,
}

a.spells["Summon Felhunter (Sacrifice down)"] = {
	ID = 691--[[Summon Felhunter]],
	NotWhileMoving = true,
	Check = function(z)
		return x.ActiveEnemy and not x.PetAlive and s.HasTalent(108503--[[Grimoire of Sacrifice]]) and not s.Buff(108503--[[Grimoire of Sacrifice]], "player")
	end,
	FlashID = {691--[[Summon Felhunter]], 10--[[Summon Demon]]}
}

a.spells["Summon Felguard (Sacrifice down)"] = {
	ID = 30146--[[Summon Felguard]],
	NotWhileMoving = true,
	Check = function(z)
		return x.ActiveEnemy and not x.PetAlive and s.HasTalent(108503--[[Grimoire of Sacrifice]]) and not s.Buff(108503--[[Grimoire of Sacrifice]], "player")
	end,
	FlashID = {30146--[[Summon Felguard]], 10--[[Summon Demon]]}
}

a.spells["Summon Doomguard"] = {
	ID = 18540--[[Summon Doomguard]],
	Check = function(z)
		return x.ActiveEnemy
	end,
	FlashColor = "Purple",
	NotWhileMoving = false,
	EnemyTargetNeeded = true,
	Continue = true,
}

a.spells["Summon Terrorguard"] = {
	ID = 112927--[[Summon Terrorguard]],
	Check = function(z)
		return x.ActiveEnemy
	end,
	FlashColor = "Purple",
	NotWhileMoving = false,
	EnemyTargetNeeded = true,
	Continue = true,
}

a.spells["Soul Swap with Soulburn"] = {
	ID = 86121--[[Soul Swap]],
	RequireMyBuff = 74434--[[Soulburn]],
	RequireBuffUnit = "player",
	NotWhileMoving = false,
	Check = function(z)
		return x.NoCC
	end,
	RunLast = function(z)
		x.LastSoulburn = GetTime()
	end,
	EnemyTargetNeeded = true,
}

a.spells["Soulburn with Dark Soul"] = {
	ID = 74434--[[Soulburn]],
	Buff = 74434--[[Soulburn]],
	BuffUnit = "player",
	RequireMyBuff = 113860--[[Dark Soul: Misery]],
	RequireBuffUnit = "player",
	NotWhileMoving = false,
	Check = function(z)
		return x.ActiveEnemy and (GetTime() - x.LastSoulburn) > 20
	end,
	EnemyTargetNeeded = true,
	FlashColor = "Yellow",
	NoGCD = true,
}

a.spells["Soulburn under 20%"] = {
	ID = 74434--[[Soulburn]],
	Buff = 74434--[[Soulburn]],
	BuffUnit = "player",
	NotWhileMoving = false,
	Check = function(z)
		return x.ActiveEnemy and s.HealthPercent("target") <= 20
	end,
	CheckLast = function(z)
		if x.HasPandemic then
			return s.MyDebuffDuration(30108--[[Unstable Affliction]], "target") < (a.getTotalTicks(7) * a.getHastedTickTime(2) * 0.5) or s.MyDebuffDuration(172--[[Corruption]], "target") < (a.getTotalTicks(9) * a.getHastedTickTime(2) * 0.5) or s.MyDebuffDuration(980--[[Agony]], "target") < (a.getTotalTicks(12) * a.getHastedTickTime(2) * 0.5)
		else
			return s.MyDebuffDuration(30108--[[Unstable Affliction]], "target") <= x.DoTRefresh or s.MyDebuffDuration(172--[[Corruption]], "target") <= x.DoTRefresh or s.MyDebuffDuration(980--[[Agony]], "target") <= x.DoTRefresh
		end
	end,
	EnemyTargetNeeded = true,
	FlashColor = "Yellow",
	NoGCD = true,
}

a.spells["Haunt"] = {
	ID = 48181--[[Haunt]],
	MyDebuff = 48181--[[Haunt]],
--	RequireMyDebuff1 = 980--[[Agony]],
--	RequireMyDebuff2 = 172--[[Corruption]],
--	RequireMyDebuff3 = 30108--[[Unstable Affliction]],
	NotWhileMoving = true,
	Check = function(z)
		z.EarlyRefresh = (s.LastSpellTravelTime(48181--[[Haunt]], "target") or 0) + s.CastTime(48181--[[Haunt]]) - 0.1
		return x.NoCC and not s.SpellOrAuraDelay(48181--[[Haunt]], "target", true) and s.Power("player", SPELL_POWER_SOUL_SHARDS ) > 1
	end,
	EnemyTargetNeeded = true,
}

a.spells["Agony refresh early if not channeling"] = {
	ID = 980--[[Agony]],
	MyDebuff = 980--[[Agony]],
	NotWhileMoving = false,
	Check = function(z)
		return x.NoCC and x.HasPandemic and x.TimeToDie >= 8
	end,
	CheckLast = function(z)
		z.EarlyRefresh = a.getTotalTicks(12) * x.DoTRefresh * 0.5 - 0.1 
		return not s.Channeling(nil, "player") 
	end,
	EnemyTargetNeeded = true,
}

a.spells["Agony"] = {
	ID = 980--[[Agony]],
	MyDebuff = 980--[[Agony]],
	NotWhileMoving = false,
	Check = function(z)
		z.EarlyRefresh = x.DoTRefresh
		return x.NoCC and x.TimeToDie >= 8
	end,
	EnemyTargetNeeded = true,
}

a.spells["Corruption refresh early if not channeling"] = {
	ID = 172--[[Corruption]],
	MyDebuff = 172--[[Corruption]],
	NotWhileMoving = false,
	Check = function(z)
		return x.NoCC and x.HasPandemic and x.TimeToDie >= 6
	end,
	CheckLast = function(z)
		z.EarlyRefresh = a.getTotalTicks(9) * x.DoTRefresh * 0.5 - 0.1 
		return not s.Channeling(nil, "player")
	end,
	EnemyTargetNeeded = true,
}

a.spells["Corruption"] = {
	ID = 172--[[Corruption]],
	MyDebuff = 172--[[Corruption]],
	NotWhileMoving = false,
	Check = function(z)
		z.EarlyRefresh = x.DoTRefresh
		return x.NoCC and x.TimeToDie >= 6
	end,
	EnemyTargetNeeded = true,
}

a.spells["Corruption (Demo)"] = {
	ID = 172--[[Corruption]],
	MyDebuff = 172--[[Corruption]],
	NotWhileMoving = false,
	Check = function(z)
		z.EarlyRefresh = x.DoTRefresh
		return x.NoCC and x.TimeToDie >= 6
	end,
	CheckLast = function(z)
		return not s.Buff(103958--[[Metamorphosis]], "player")
	end,
	EnemyTargetNeeded = true,
}

a.spells["Unstable Affliction refresh early if not channeling"] = {
	ID = 30108--[[Unstable Affliction]],
	MyDebuff = 30108--[[Unstable Affliction]],
	NotWhileMoving = true,
	Check = function(z)
		return x.NoCC and x.HasPandemic and x.TimeToDie >= 5
	end,
	CheckLast = function(z)
		z.EarlyRefresh = a.getTotalTicks(7) * x.DoTRefresh * 0.5 - 0.1 
		return not s.Channeling(nil, "player")
	end,
	EnemyTargetNeeded = true,
}

a.spells["Unstable Affliction"] = {
	ID = 30108--[[Unstable Affliction]],
	MyDebuff = 30108--[[Unstable Affliction]],
	NotWhileMoving = true,
	Check = function(z)
		z.EarlyRefresh = x.DoTRefresh
		return x.NoCC and x.TimeToDie >= 5
	end,
	EnemyTargetNeeded = true,
}

a.spells["Shadowburn"] = {
	ID = 17877--[[Shadowburn]],
	Check = function(z)
		return x.NoCC
	end,
	FlashColor = "Orange",
	EnemyTargetNeeded = true,
}

a.spells["Chaos Bolt"] = {
	ID = 116858--[[Chaos Bolt]],
	Check = function(z)
		return x.NoCC and s.BuffStack(117828--[[Backdraft]], "player") < 3
	end,
	NotWhileMoving = true,
	EnemyTargetNeeded = true,
}

a.spells["Conflagrate"] = {
	ID = 17962--[[Conflagrate]],
	MyBuff = 117828--[[Backdraft]],
	BuffUnit = "player",
	Check = function(z)
		return x.NoCC
	end,
	NotWhileMoving = false,
	EnemyTargetNeeded = true,
}

a.spells["Immolate"] = {
	ID = 348--[[Immolate]],
	MyDebuff = 348--[[Immolate]],
	Check = function(z)
		return x.NoCC
	end,
	Run = function(z)
		z.EarlyRefresh = s.If(x.HasPandemic, 10.4, x.DoTRefresh)
	end,
	NotWhileMoving = true,
	EnemyTargetNeeded = true,
}

a.spells["Incinerate"] = {
	ID = 29722--[[Incinerate]],
	Check = function(z)
		return x.NoCC
	end,
	NotWhileMoving = true,
	EnemyTargetNeeded = true,
}

a.spells["Drain Soul as finisher"] = {
	ID = 1120--[[Drain Soul]],
	NotWhileMoving = true,
	Check = function(z)
		return x.NoCC
	end,
	CheckLast = function(z)
		return s.HealthPercent("target") <= 20
	end,
	EnemyTargetNeeded = true,
	FlashColor = "Orange",
}

a.spells["Life Tap if mana < 35%"] = {
	ID = 1454--[[Life Tap]],
	NotWhileMoving = false,
	Check = function(z)
		return x.ActiveEnemy
	end,
	CheckLast = function(z)
		return s.PowerPercent("player", SPELL_POWER_MANA) < 35
	end,
}

a.spells["Life Tap while moving if mana < 80%"] = {
	ID = 1454--[[Life Tap]],
	NotWhileMoving = false,
	Check = function(z)
		return x.ActiveEnemy and s.Moving("player") and s.PowerPercent("player", SPELL_POWER_MANA) < 80
	end,
	CheckLast = function(z)
		return s.PowerPercent("player", SPELL_POWER_MANA) < s.HealthPercent("target")
	end,
}

a.spells["Life Tap if mana < 50%"] = {
	ID = 1454--[[Life Tap]],
	NotWhileMoving = false,
	Check = function(z)
		return x.ActiveEnemy
	end,
	CheckLast = function(z)
		return s.PowerPercent("player", SPELL_POWER_MANA) < 50
	end,
}

a.spells["Malefic Grasp"] = {
	ID = 103103--[[Malefic Grasp]],
	NotWhileMoving = true,
	Check = function(z)
		return x.NoCC
	end,
	EnemyTargetNeeded = true,
}

a.spells["Fel Flame while moving"] = {
	ID = 77799--[[Fel Flame]],
	NotWhileMoving = false,
	Check = function(z)
		return x.NoCC and s.Moving("player")
	end,
	EnemyTargetNeeded = true,
}

a.spells["Life Tap"] = {
	ID = 1454--[[Life Tap]],
	NotWhileMoving = false,
	Check = function(z)
		return x.ActiveEnemy
	end,
	NoStopChannel = true,
}

a.spells["Command Demon - Wrathstorm"] = {
	ID = 119915--[[Wrathstorm]],
	Override = function(z)
		return true
	end,
	NoGCD = true,
	FlashColor = "Yellow",
	FlashID = {119915--[[Wrathstorm]], 119898--[[Command Demon]]},
}

a.spells["Command Demon - Felstorm"] = {
	ID = 119914--[[Felstorm]],
	Override = function(z)
		return true
	end,
	NoGCD = true,
	FlashColor = "Yellow",
	FlashID = {119914--[[Felstorm]], 119898--[[Command Demon]]},
}

a.spells["Wrathstorm"] = {
	Type = "pet",
	ID = 115831--[[Wrathstorm]],
	Check = function(z)
		return x.PetAlive and x.PetActiveEnemy
	end,
	RunLast = function(z)
		a.Flash("Command Demon - Wrathstorm")
	end,
	FlashColor = "Yellow",
	NoGCD = true,
}

a.spells["Felstorm"] = {
	Type = "pet",
	ID = 89751--[[Felstorm]],
	Check = function(z)
		return x.PetAlive and x.PetActiveEnemy
	end,
	RunLast = function(z)
		a.Flash("Command Demon - Felstorm")
	end,
	FlashColor = "Yellow",
	NoGCD = true,
}

a.spells["Metamorphosis"] = {
	Type = "form",
	ID = 103958--[[Metamorphosis]],
	Buff = 103958--[[Metamorphosis]],
	BuffUnit = "player",
	NotIfActive = true,
	FlashBlink = true,
	Check = function(z)
		return x.ActiveEnemy
	end,
	CheckLast = function(z)
		return s.Buff(113861--[[Dark Soul: Knowledge]], "player") or (x.CorruptionRemaining > 0 and x.CorruptionRemaining < 6) or (s.Power("player", SPELL_POWER_DEMONIC_FURY) >= min(900, x.TimeToDie * 30))
	end,
	FlashColor = "Yellow",
	NoGCD = true,
}

a.spells["Cancel Metamorphosis"] = {
	Type = "form",
	ID = 103958--[[Metamorphosis]],
	FlashBlink = true,
	Override = function(z)
		return x.ActiveEnemy and s.Buff(103958--[[Metamorphosis]], "player") and ((x.CorruptionRemaining > 20) or (x.CorruptionRemaining == 0)) and not s.Buff(113861--[[Dark Soul: Knowledge]], "player") and s.Power("player", SPELL_POWER_DEMONIC_FURY) <= 750 and x.TimeToDie > 30
	end,
	FlashColor = "Pink",	
	NoGCD = true,
	Continue = true,
}

a.spells["Imp Swarm"] = {
	ID = 104316--[[Imp Swarm]],
	Check = function(z)
		return x.NoCC
	end,
	EnemyTargetNeeded = true,
}

a.spells["Hand of Gul'dan"] = {
	ID = 105174--[[Hand of Gul'dan]],
	MyDebuff = 47960--[[Shadowflame]],
	Check = function(z)
		z.EarlyRefresh = (s.LastSpellTravelTime(105174--[[Hand of Gul'dan]], "target") or 0) + s.CastTime(686--[[Shadow Bolt]])
		return x.NoCC and not s.SpellOrAuraDelay(105174--[[Hand of Gul'dan]], "target", any)
	end,
	CheckLast = function(z)
		return not s.Buff(103958--[[Metamorphosis]], "player")
	end,
	EnemyTargetNeeded = true,
}

a.spells["Soul Fire"] = {
	ID = 6353--[[Soul Fire]],
	RequireMyBuff = 122355--[[Molten Core]],
	RequireBuffUnit = "player",
	Check = function(z)
		if s.HealthPercent("target") < 25 then
			z.FlashColor = "Orange"
		else
			z.FlashColor = "Yellow"
		end
		return x.NoCC
	end,
	EnemyTargetNeeded = true,
}

a.spells["Doom"] = {
	ID = 124913--[[Metamorphosis: Doom]],
	MyDebuff = 603--[[Doom]],
	RequireBuff = 103958--[[Metamorphosis]],
	RequireBuffUnit = "player",
	Check = function(z)
		if s.Buff(113861--[[Dark Soul: Knowledge]], "player") then
			z.EarlyRefresh = (x.TotalDoomTicks - 2) * x.HastedDoomTick - 0.1
		else
			z.EarlyRefresh = s.If(x.HasPandemic, (x.TotalDoomTicks * x.HastedDoomTick * 0.5) - 0.1, x.HastedDoomTick - 0.1)
		end
		return x.NoCC and x.TimeToDie >= 30 
	end,
	FlashID = { 124913--[[Metamorphosis: Doom]], 603--[[Doom]], 172--[[Corruption]] },
	EnemyTargetNeeded = true,
}

a.spells["Touch of Chaos if Corruption < 20sec"] = {
	ID = 686--[[Shadow Bolt]],
	RequireBuff = 103958--[[Metamorphosis]],
	RequireBuffUnit = "player",
	Check = function(z)
		return x.NoCC and x.CorruptionRemaining > 0 and x.CorruptionRemaining < 20
	end,
	EnemyTargetNeeded = true,
}

a.spells["Touch of Chaos"] = {
	ID = 686--[[Shadow Bolt]],
	RequireBuff = 103958--[[Metamorphosis]],
	RequireBuffUnit = "player",
	Check = function(z)
		return x.NoCC
	end,
	EnemyTargetNeeded = true,
}

a.spells["Shadow Bolt"] = {
	ID = 686--[[Shadow Bolt]],
	Check = function(z)
		return x.NoCC
	end,
	CheckLast = function(z)
		return not s.Buff(103958--[[Metamorphosis]], "player")
	end,
	EnemyTargetNeeded = true,
}

a.spells["Potion with Bloodlust or Boss HP <= 20"] = {
	Type = "item",
	ID = 76093--[[Potion of the Jade Serpent]],
	Buff = {105702--[[Potion of the Jade Serpent]], 79476--[[Volcanic Power]]},
	EnemyTargetNeeded = true,
	NoGCD = true,
	Check = function(z)
		return x.ShouldTemporaryBuff and x.ActiveEnemy and s.Boss()
	end,
	CheckLast = function(z)
		return a.Bloodlust() or (s.HealthPercent("target") <= 20)
	end,
	FlashID = {76093--[[Potion of the Jade Serpent]], 58091--[[Volcanic Potion]]},
	FlashColor = "Yellow",
	Continue = true,
}

a.spells["Carrion Swarm"] = {
	ID = 103967--[[Carrion Swarm]],
	Check = function(z)
		return x.NoCC
	end,
	CheckLast = function(z)
		return not s.Boss("target")
	end,
	Interrupt = true,
	FlashColor = "Aqua",
	EnemyTargetNeeded = true,
}

a.spells["Banish"] = {
	ID = 710--[[Banish]],
	EnemyTargetNeeded = 1,
	Check = function(z)
		return x.NoCC
	end,
	CheckLast = function(z)
		return not s.Boss("target")
	end,
	NotWhileMoving = true,
	FlashColor = "Pink",
}

a.spells["Create Healthstone"] = {
	ID = 6201--[[Create Healthstone]],
	Check = function(z)
		return x.ShouldTemporaryBuff 
	end,
	CheckLast = function(z)
		return GetItemCount(5512--[[Healthstone]], true) == 0 and not s.InGroup()
	end,
	NotWhileMoving = true,
	FlashColor = "Green",
}

a.spells["Create Soulwell"] = {
	ID = 29893--[[Create Soulwell]],
	Check = function(z)
		return x.ShouldTemporaryBuff 
	end,
	CheckLast = function(z)
		return GetItemCount(5512--[[Healthstone]], true) == 0 and s.InGroup()
	end,
	NotWhileMoving = true,
	FlashColor = "Green",
}

a.spells["Soulshatter"] = {
	ID = 29858--[[Soulshatter]],
	EnemyTargetNeeded = 1,
	FlashColor = "Red",
	Check = function(z)
		return x.ThreatPercent >= 80 and s.InGroup()
	end,
}